@@ -1,6 +1,7 @@
 # Copyright 2019 Cognite AS
 import logging
 import os
+import sys
 import time
 
 import grpc
@@ -27,7 +28,8 @@ if not os.getenv("READ_THE_DOCS"):
     from cognite.seismic._api.seismics import SeismicAPI
     from cognite.seismic._api.seismicstores import SeismicStoreAPI
     from cognite.seismic._api.volume_seismic import VolumeSeismicAPI
-
+    from cognite.seismic._api.file_seismic import FileSeismicAPI
+    from cognite.seismic.data_classes.errors import _from_grpc_error, SeismicServiceError
 
 # The maximum number of retries
 _MAX_RETRIES_BY_CODE = {
@@ -44,13 +46,7 @@ _MAX_SLEEPING = 5.0
 logger = logging.getLogger(__name__)
 
 
-class RetriesExceeded(Exception):
-    """docstring for RetriesExceeded"""
-
-    pass
-
-
-def retry(f, transactional=False):
+def with_retry(f, transactional=False):
     def wraps(*args, **kwargs):
         retries = 0
         while True:
@@ -64,7 +60,10 @@ def retry(f, transactional=False):
                     raise
 
                 if retries > max_retries:
-                    raise RetriesExceeded(e)
+                    e = SeismicServiceError()
+                    e.status = grpc.StatusCode.UNAVAILABLE
+                    e.message = "maximum number of retries exceeded"
+                    raise e
 
                 backoff = min(_MIN_SLEEPING * 2 ** retries, _MAX_SLEEPING)
                 logger.info("sleeping %r for %r before retrying failed request...", backoff, code)
@@ -75,15 +74,39 @@ def retry(f, transactional=False):
     return wraps
 
 
-def retrying_stub_methods(obj):
-    for key, attr in obj.__dict__.items():
-        if (
-            isinstance(attr, _UnaryUnaryMultiCallable)
-            or isinstance(attr, _StreamStreamMultiCallable)
-            or isinstance(attr, _UnaryStreamMultiCallable)
-            or isinstance(attr, _SingleThreadedUnaryStreamMultiCallable)
-        ):
-            setattr(obj, key, retry(attr))
+def decorate_with_retries(*args):
+    for obj in args:
+        for key, attr in obj.__dict__.items():
+            if (
+                isinstance(attr, _UnaryUnaryMultiCallable)
+                or isinstance(attr, _StreamStreamMultiCallable)
+                or isinstance(attr, _UnaryStreamMultiCallable)
+                or isinstance(attr, _SingleThreadedUnaryStreamMultiCallable)
+            ):
+                setattr(obj, key, with_retry(attr))
+
+
+def with_wrapped_rpc_errors(f):
+    def wraps(*args, **kwargs):
+        try:
+            return f(*args, **kwargs)
+        except _InactiveRpcError as e:
+            # strip messy internal grpc stack that confuses users
+            tb = sys.exc_info()[2]
+            while True:
+                if tb.tb_next == None:
+                    break
+                else:
+                    tb = tb.tb_next
+            raise _from_grpc_error(e).with_traceback(tb) from None
+
+    return wraps
+
+
+def decorate_with_wrapped_rpc_errors(*args):
+    for obj in args:
+        for key, attr in obj.__dict__.items():
+            setattr(obj, key, with_wrapped_rpc_errors(attr))
 
 
 class CogniteSeismicClient:
@@ -91,11 +114,11 @@ class CogniteSeismicClient:
     Main class for the seismic client
     """
 
-    def __init__(
-        self, api_key=None, base_url=None, port=None, experimental=None, custom_root_cert=None, *, insecure=False
-    ):
+    def __init__(self, api_key=None, base_url=None, port=None, custom_root_cert=None, *, insecure=False, project=None):
         # configure env
         self.api_key = api_key or os.getenv("COGNITE_API_KEY")
+        self.project = project or os.getenv("COGNITE_PROJECT")
+
         if self.api_key is None or self.api_key == "":
             raise ValueError(
                 "You have either not passed an api key or not set the COGNITE_API_KEY environment variable."
@@ -104,8 +127,13 @@ class CogniteSeismicClient:
         self.port = port or "443"
         self.url = self.base_url + ":" + str(self.port)
         self.metadata = [("api-key", self.api_key)]
-        if experimental is not None:
-            self.metadata.append(("use-experimental", "true"))
+
+        if self.project is not None:
+            self.metadata.append(("cdf-project-name", self.project))
+        else:
+            logger.warning(
+                "Undefined project name is deprecated behaviour. Please set the project name with CogniteSeismicClient(project='') or with environment variable COGNITE_PROJECT=''"
+            )
 
         # start the connection
         options = [
@@ -123,9 +151,9 @@ class CogniteSeismicClient:
         self.query = query_serv.QueryStub(channel)
         self.ingestion = ingest_serv.IngestStub(channel)
         self.seismicstub = seismic_serv.SeismicAPIStub(channel)
-        retrying_stub_methods(self.query)
-        retrying_stub_methods(self.ingestion)
-        retrying_stub_methods(self.seismicstub)
+
+        decorate_with_retries(self.query, self.ingestion, self.seismicstub)
+        decorate_with_wrapped_rpc_errors(self.query, self.ingestion, self.seismicstub)
 
         self.survey = SurveyAPI(self.query, self.ingestion, self.metadata)
         self.file = FileAPI(self.query, self.ingestion, self.metadata)
@@ -138,3 +166,4 @@ class CogniteSeismicClient:
         self.partition = PartitionAPI(self.seismicstub, self.ingestion, self.metadata)
         self.seismics = SeismicAPI(self.seismicstub, self.ingestion, self.metadata)
         self.seismicstores = SeismicStoreAPI(self.seismicstub, self.ingestion, self.metadata)
+        self.file_seismic = FileSeismicAPI(self.seismicstub, self.ingestion, self.metadata)
